# Zach Gildersleeve
# zgildersleeve@gmail.com
# CS6630 - Scientific Visualization
# Project 2
# Oct. 17, 2006
#
# biColor.tcl - This script generates a multivariate colormap lookup
# table through two nested for loops, which takes advantage of
# data ordering, as described in the project report and the assignment
# specifications.  The necessary datasets are read in as structured points,
# mapped with the lookup table, and rendered.
#
# A simple GUI allows the user to switch between datasets, but the
# colormap must be changed by hand in the code.


catch {load vtktcl}
set VTK_TCL ".."
set VTK_DATA "../vtkdata"

set bits 8
set name -$bits

##########
# Create colormap, and populate with colors through
# different sets of commands.  Comment in/out blocks to
# generate different colormaps.
#
set power [expr int( pow(2, $bits))]
vtkLookupTable lut
	lut SetNumberOfColors [expr $power * $power]
	lut SetTableRange 0 [expr $power * $power - 1]

for {set i 0} {$i < $power} {incr i} {
	for {set j 0} {$j < $power} {incr j} {

	# BWYK
		set R [expr $j / $power.0]
		set G [expr $j / $power.0]
		set B [expr $i / $power.0]

	# BWYK
#		set R [expr $j / $power.0]
#		set G 0
#		set B [expr $i / $power.0]

	# B->W->Y->W->B->W->Y->W->B around K (BiK)
#		set R [expr abs(($j - ($power / 2.0)) / ($power /  2.0))]
#		set G [expr abs(($j - ($power / 2.0)) / ($power /  2.0))]
#		set B [expr abs(($i - ($power / 2.0)) / ($power /  2.0))]

	# Y->K->B->K->Y->K->B->K->Y around W (BiW)
#		set R [expr 1 - abs(($j - ($power / 2.0)) / ($power /  2.0))]
#		set G [expr 1 - abs(($j - ($power / 2.0)) / ($power /  2.0))]
#		set B [expr 1 - abs(($i - ($power / 2.0)) / ($power /  2.0))]

		lut SetTableValue [expr $power * $i + $j] $R $G $B 1.0
	}
}

##########
# Read in Mt. Hood structured points dataset
#
vtkStructuredPointsReader mtHood_reader
	mtHood_reader SetFileName $VTK_DATA/MtHood$name.vtk

vtkImageMapToColors mtHood_image
	mtHood_image SetInput [mtHood_reader GetOutput]
	mtHood_image SetLookupTable lut
	mtHood_image SetOutputFormatToRGB

##########
# Read in non-radically symmetrical brain data
#
vtkStructuredPointsReader brain_reader
	brain_reader SetFileName $VTK_DATA/PD-T2$name.vtk

vtkImageMapToColors brain_image
	brain_image SetInput [brain_reader GetOutput]
	brain_image SetLookupTable lut
	brain_image SetOutputFormatToRGB

##########
# Read in radically symmetrical brain data
#
vtkStructuredPointsReader radBrain_reader
	radBrain_reader SetFileName $VTK_DATA/tb-ax$name.vtk

vtkImageMapToColors radBrain_image
	radBrain_image SetInput [radBrain_reader GetOutput]
	radBrain_image SetLookupTable lut
	radBrain_image SetOutputFormatToRGB

##########
# Read in colormap disc data
#
vtkStructuredPointsReader disc_reader
	disc_reader SetFileName $VTK_DATA/disc$name.vtk

vtkImageMapToColors disc_image
	disc_image SetInput [disc_reader GetOutput]
	disc_image SetLookupTable lut
	disc_image SetOutputFormatToRGB



vtkImageActor actor
	actor SetInput [disc_image GetOutput]



##########
# Create the Renderer and Windows
#
vtkRenderer ren1
vtkRenderWindow renWin
	renWin AddRenderer ren1
vtkRenderWindowInteractor iren
	iren SetRenderWindow renWin

ren1 AddActor actor
ren1 SetBackground 0.5 0.5 0.5
ren1 ResetCamera

renWin SetSize 512 512

renWin Render

##########
# Create the GUI elements and pack the GUI window
#
button .quit -text "Quit" -command {exit}

frame .imageSelect -relief raised -borderwidth 5
frame .imageSelect.buttons
label .imageSelect.buttons.l1 -text "Select Image Data"
radiobutton .imageSelect.buttons.r1 -text "Mt. Hood" -command setImage -var imageSelection -value mtHood
radiobutton .imageSelect.buttons.r2 -text "Brain MRI" -command setImage -var imageSelection -value brain
radiobutton .imageSelect.buttons.r3 -text "Brain DT-MRI" -command setImage -var imageSelection -value radical
radiobutton .imageSelect.buttons.r4 -text "disc" -command setImage -var imageSelection -value disc

set imageSelection disc

grid .quit -row 0 -column 0

grid .imageSelect.buttons -row 0 -column 0
grid .imageSelect.buttons.l1 -row 1 -column 0
grid .imageSelect.buttons.r1 -row 2 -column 0
grid .imageSelect.buttons.r2 -row 3 -column 0
grid .imageSelect.buttons.r3 -row 4 -column 0
grid .imageSelect.buttons.r4 -row 5 -column 0

grid .imageSelect -row 1 -column 0

###
# Sets the image dataset to draw.  All images use the same
# colormap and renderer.
#
proc setImage {args} {
	global imageSelection
	ren1 RemoveActor actor

	switch $imageSelection	{
		"mtHood"	{actor SetInput [mtHood_image GetOutput]}
		"brain"		{actor SetInput [brain_image GetOutput]}
		"radical"	{actor SetInput [radBrain_image GetOutput]}
		"disc"		{actor SetInput [disc_image GetOutput]}
	}
	ren1 AddActor actor
	ren1 ResetCamera
	renWin Render
}

