# Zach Gildersleeve
# zgildersleeve@gmail.com
# CS6630 - Scientific Visualization
# Project 2
# Oct. 17, 2006
#
# uniColor.tcl - This script creates a GUI that allows the user to set
# both the start and end point of either a RGB or HSV univariate colormap,
# and apply that to the user selected image.  Functionality is also
# provided in which either the 2D representation of the image dataset or a 3D
# warped version can be rendered.

catch {load vtktcl}
set VTK_TCL ".."
set VTK_DATA "../vtkdata"

##########
# Create and initialize the color lookup tables, and
# populate the RGB colormap with black.
#
vtkLookupTable RGBlut
  	RGBlut SetNumberOfColors 256
 	 RGBlut SetTableRange 0 255
  	for {set i 0} {$i <= 255} {incr i} {
    		RGBlut SetTableValue $i 0 0 0 1.0
  	}

vtkLookupTable HSVlut
	HSVlut SetNumberOfColors 256
	HSVlut SetTableRange 0 255
	HSVlut SetAlphaRange 1.0 1.0
	
##########
# Read in Mt. Hood data, create Actor for 3D version and Actor2D for 2D version
# While it is possible to create an actor directly from vtkImageViewer by
# calling [viewer GetActor], updating the colormaps prove to be more
# difficult with multiple actors available. This pipeline is based on
# the workflow established in project1
#
vtkPNMReader reader_mtHood
	reader_mtHood SetFileName $VTK_DATA/MtHood.pgm

vtkImageMagnitude magnitude_mtHood
	magnitude_mtHood SetInput [reader_mtHood GetOutput]

vtkImageDataGeometryFilter geometry_mtHood
	geometry_mtHood SetInput [magnitude_mtHood GetOutput]

vtkWarpScalar warp_mtHood
	warp_mtHood SetInput [geometry_mtHood GetOutput]
	warp_mtHood SetScaleFactor 0.25

vtkMergeFilter merge_mtHood
	merge_mtHood SetGeometry [warp_mtHood GetOutput]
	merge_mtHood SetScalars [reader_mtHood GetOutput]

# what values do the low and high points correspond to?
vtkElevationFilter elevation_mtHood
	elevation_mtHood SetInput [merge_mtHood GetOutput]
	elevation_mtHood SetLowPoint 0 0 0
	elevation_mtHood SetHighPoint 0 0 50

vtkDataSetMapper mapper_3D_mtHood
	mapper_3D_mtHood SetInput [elevation_mtHood GetOutput]
	mapper_3D_mtHood SetLookupTable RGBlut

vtkPolyDataMapper2D mapper_2D_mtHood
	mapper_2D_mtHood SetInput [elevation_mtHood GetOutput]
	mapper_2D_mtHood SetLookupTable RGBlut

vtkActor2D actor_2D_mtHood
	actor_2D_mtHood SetMapper mapper_2D_mtHood
	[actor_2D_mtHood GetPositionCoordinate] SetCoordinateSystemToNormalizedDisplay
	[actor_2D_mtHood GetPositionCoordinate] SetValue 0.25 0.25

vtkActor actor_3D_mtHood
	actor_3D_mtHood SetMapper mapper_3D_mtHood

##########
# Read in thorax data, create Actor for 3D version and Actor2D for 2D version.
# This pipeline is based on the workflow established in project1, and uses
# the points.vtk data generated by writepoints.tcl.  Commented out
# is a butterfly subdivision filter that can reduce visual artifacts in the
# thorax data, as explained in the report.
#
vtkPolyDataReader reader_thorax
	reader_thorax SetFileName thoraxPoints.vtk
	reader_thorax OpenVTKFile

vtkDelaunay2D delaunay_thorax
	delaunay_thorax SetInput [reader_thorax GetOutput]
	delaunay_thorax SetTolerance 0.001
#	delaunay_thorax SetAlpha 0
	delaunay_thorax SetAlpha 18.0
	delaunay_thorax Update

vtkElevationFilter elevation_thorax
	elevation_thorax SetInput [delaunay_thorax GetOutput]
	elevation_thorax SetLowPoint 0 0 -25
	elevation_thorax SetHighPoint 0 0 25

#vtkLinearSubdivisionFilter butterfly_thorax
#	butterfly_thorax SetInput [elevation_thorax GetOutput]
#	butterfly_thorax SetNumberOfSubdivisions 5

vtkDataSetMapper mapper_3D_thorax
	mapper_3D_thorax SetInput [elevation_thorax GetOutput]
#	mapper_3D_thorax SetInput [butterfly_thorax GetOutput]
	mapper_3D_thorax SetLookupTable RGBlut

vtkActor actor_3D_thorax
	actor_3D_thorax SetMapper mapper_3D_thorax

vtkPolyDataMapper2D mapper_2D_thorax
	mapper_2D_thorax SetInput [elevation_thorax GetOutput]
	mapper_2D_thorax SetLookupTable RGBlut

vtkActor2D actor_2D_thorax
	actor_2D_thorax SetMapper mapper_2D_thorax
	[actor_2D_thorax GetPositionCoordinate] SetCoordinateSystemToNormalizedDisplay
	[actor_2D_thorax GetPositionCoordinate] SetValue 0.5 0.5

##########
# Read in dem data, create Actor for 3D version and Actor2D for 2D version.
# This pipeline is based on the dem.tcl example, as well as project1.  There
# is no 2D image version of this dataset. 
#
vtkDEMReader reader_dem
	reader_dem SetFileName salt_lake_city-e.dem
	reader_dem Update

vtkImageShrink3D shrink_dem
	shrink_dem SetShrinkFactors 256 256 1
	shrink_dem SetInput [reader_dem GetOutput]

vtkImageDataGeometryFilter geometry_dem
	geometry_dem SetInput [reader_dem GetOutput]
	geometry_dem ReleaseDataFlagOn

vtkWarpScalar warp_dem
	warp_dem SetInput [geometry_dem GetOutput]
	warp_dem SetScaleFactor 5

# here the high and low points are given by the data
vtkElevationFilter elevation_dem
	elevation_dem SetInput [warp_dem GetOutput]
	elevation_dem SetLowPoint 0 0 [expr 5 * [lindex [reader_dem GetElevationBounds] 0]]
	elevation_dem SetHighPoint 0 0 [expr 5 * [lindex [reader_dem GetElevationBounds] 1]]
eval elevation_dem SetScalarRange [expr 5 * [lindex [reader_dem GetElevationBounds] 0]] [expr 5 * [lindex [reader_dem GetElevationBounds] 1]]

vtkCastToConcrete toPoly_dem
	toPoly_dem SetInput [elevation_dem GetOutput]

vtkPolyDataNormals normals_dem
    	normals_dem SetInput [toPoly_dem GetPolyDataOutput]
    	normals_dem SetFeatureAngle 60
    	normals_dem ConsistencyOff
   	normals_dem SplittingOff
   	normals_dem ReleaseDataFlagOn

vtkPolyDataMapper mapper_3D_dem
    	mapper_3D_dem SetInput [normals_dem GetOutput]
eval mapper_3D_dem SetScalarRange [expr 5 * [lindex [reader_dem GetElevationBounds] 0]] [expr 5 * [lindex [reader_dem GetElevationBounds] 1]]
    	mapper_3D_dem SetLookupTable RGBlut
    	mapper_3D_dem ImmediateModeRenderingOn
	mapper_3D_dem Update

vtkActor actor_3D_dem
	actor_3D_dem SetMapper mapper_3D_dem

##########
# Create the Renderer and Windows
#
vtkRenderer ren1
vtkRenderWindow renWin
	renWin AddRenderer ren1
vtkRenderWindowInteractor iren
	iren SetRenderWindow renWin

ren1 AddActor actor_2D_mtHood
ren1 SetBackground 0.5 0.5 0.5
ren1 ResetCamera

renWin SetSize 512 512

renWin Render


#########
# Set up the GUI widgets
#
frame .fRGB -relief raised -borderwidth 5
frame .fRGB.sliders
label .fRGB.sliders.lstart1 -text "RGB Start Point"
label .fRGB.sliders.lstart2 -text "Red"
scale .fRGB.sliders.sstart1 -from 0 -to 255 -length 255 -resolution 5 -showvalue true -orient horizontal -command setRGB
label .fRGB.sliders.lstart3 -text "Green"
scale .fRGB.sliders.sstart2 -from 0 -to 255 -length 255 -resolution 5 -showvalue true -orient horizontal -command setRGB
label .fRGB.sliders.lstart4 -text "Blue"
scale .fRGB.sliders.sstart3 -from 0 -to 255 -length 255 -resolution 5 -showvalue true -orient horizontal -command setRGB
label .fRGB.sliders.lend1 -text "RGB End Point"
label .fRGB.sliders.lend2 -text "Red"
scale .fRGB.sliders.send1 -from 0 -to 255 -length 255 -resolution 5 -showvalue true -orient horizontal -command setRGB
label .fRGB.sliders.lend3 -text "Green"
scale .fRGB.sliders.send2 -from 0 -to 255 -length 255 -resolution 5 -showvalue true -orient horizontal -command setRGB
label .fRGB.sliders.lend4 -text "Blue"
scale .fRGB.sliders.send3 -from 0 -to 255 -length 255 -resolution 5 -showvalue true -orient horizontal -command setRGB

frame .fHSV -relief raised -borderwidth 5
frame .fHSV.sliders
label .fHSV.sliders.lstart1 -text "HSV Start Point"
label .fHSV.sliders.lstart2 -text "Hue"
scale .fHSV.sliders.sstart1 -from 0.0 -to 1.0 -length 255 -resolution 0.01 -showvalue true -orient horizontal -command setHSV
label .fHSV.sliders.lstart3 -text "Saturation"
scale .fHSV.sliders.sstart2 -from 0.0 -to 1.0 -length 255 -resolution 0.01 -showvalue true -orient horizontal -command setHSV
label .fHSV.sliders.lstart4 -text "Value"
scale .fHSV.sliders.sstart3 -from 0.0 -to 1.0 -length 255 -resolution 0.01 -showvalue true -orient horizontal -command setHSV
label .fHSV.sliders.lend1 -text "HSV End Point"
label .fHSV.sliders.lend2 -text "Hue"
scale .fHSV.sliders.send1 -from 0.0 -to 1.0 -length 255 -resolution 0.01 -showvalue true -orient horizontal -command setHSV
label .fHSV.sliders.lend3 -text "Saturation"
scale .fHSV.sliders.send2 -from 0.0 -to 1.0 -length 255 -resolution 0.01 -showvalue true -orient horizontal -command setHSV
label .fHSV.sliders.lend4 -text "Value"
scale .fHSV.sliders.send3 -from 0.0 -to 1.0 -length 255 -resolution 0.01 -showvalue true -orient horizontal -command setHSV

frame .lutSelect -relief raised -borderwidth 5
frame .lutSelect.buttons
label .lutSelect.buttons.l1 -text "Set Table Colorspace"
radiobutton .lutSelect.buttons.r1 -text "RGB" -command setLUT -var colorSpace -value RGB
radiobutton .lutSelect.buttons.r2 -text "HSV" -command setLUT -var colorSpace -value HSV

frame .imageSelect -relief raised -borderwidth 5
frame .imageSelect.buttons
label .imageSelect.buttons.l1 -text "Select Image Data"
radiobutton .imageSelect.buttons.r1 -text "Mt. Hood" -command setImage -var imageSelection -value mtHood
radiobutton .imageSelect.buttons.r2 -text "Thorax" -command setImage -var imageSelection -value thorax
radiobutton .imageSelect.buttons.r3 -text "DEM (3D only)" -command setImage -var imageSelection -value dem

frame .heightSwitch -relief raised -borderwidth 5
frame .heightSwitch.buttons
label .heightSwitch.buttons.l1 -text "Select Data Dimension"
radiobutton .heightSwitch.buttons.r1 -text "2D" -command setImage -var heightSwitch -value two
radiobutton .heightSwitch.buttons.r2 -text "3D" -command setImage -var heightSwitch -value three

frame .f1
frame .f1.keepMe
button .f1.keepMe.resetSliders -text "Reset Colorspace" -command {resetDefaults $colorSpace}
button .f1.keepMe.quit -text "Quit" -command {exit}

#########
# Set the defaults for the GUI
#
set colorSpace RGB
set imageSelection mtHood
set heightSwitch two
.fRGB.sliders.send1 set 255
.fRGB.sliders.send2 set 255
.fRGB.sliders.send3 set 255

##########
# Arrange the widgets and GUI elements in each
# respective frame
#
grid .fRGB.sliders -row 0 -column 0
grid .fRGB.sliders.lstart1 -row 0 -column 0
grid .fRGB.sliders.lstart2 -row 1 -column 0
grid .fRGB.sliders.sstart1 -row 1 -column 1
grid .fRGB.sliders.lstart3 -row 2 -column 0
grid .fRGB.sliders.sstart2 -row 2 -column 1
grid .fRGB.sliders.lstart4 -row 3 -column 0
grid .fRGB.sliders.sstart3 -row 3 -column 1

grid .fRGB.sliders.lend1 -row 4 -column 0
grid .fRGB.sliders.lend2 -row 5 -column 0
grid .fRGB.sliders.send1 -row 5 -column 1
grid .fRGB.sliders.lend3 -row 6 -column 0
grid .fRGB.sliders.send2 -row 6 -column 1
grid .fRGB.sliders.lend4 -row 7 -column 0
grid .fRGB.sliders.send3 -row 7 -column 1

grid .lutSelect.buttons -row 0 -column 0
grid .lutSelect.buttons.l1 -row 0 -column 0 -sticky w
grid .lutSelect.buttons.r1 -row 1 -column 0 -sticky w
grid .lutSelect.buttons.r2 -row 2 -column 0 -sticky w

grid .imageSelect.buttons -row 0 -column 0
grid .imageSelect.buttons.l1 -row 0 -column 0 -sticky w
grid .imageSelect.buttons.r1 -row 1 -column 0 -sticky w
grid .imageSelect.buttons.r2 -row 2 -column 0 -sticky w
grid .imageSelect.buttons.r3 -row 3 -column 0 -sticky w

grid .heightSwitch.buttons -row 0 -column 0
grid .heightSwitch.buttons.l1 -row 0 -column 0 -sticky w
grid .heightSwitch.buttons.r1 -row 1 -column 0 -sticky w
grid .heightSwitch.buttons.r2 -row 2 -column 0 -sticky w

grid .fHSV.sliders -row 0 -column 0
grid .fHSV.sliders.lstart1 -row 0 -column 0
grid .fHSV.sliders.lstart2 -row 1 -column 0
grid .fHSV.sliders.sstart1 -row 1 -column 1
grid .fHSV.sliders.lstart3 -row 2 -column 0
grid .fHSV.sliders.sstart2 -row 2 -column 1
grid .fHSV.sliders.lstart4 -row 3 -column 0
grid .fHSV.sliders.sstart3 -row 3 -column 1

grid .fHSV.sliders.lend1 -row 4 -column 0
grid .fHSV.sliders.lend2 -row 5 -column 0
grid .fHSV.sliders.send1 -row 5 -column 1
grid .fHSV.sliders.lend3 -row 6 -column 0
grid .fHSV.sliders.send2 -row 6 -column 1
grid .fHSV.sliders.lend4 -row 7 -column 0
grid .fHSV.sliders.send3 -row 7 -column 1

grid .f1.keepMe -row 0 -column 0
grid .f1.keepMe.resetSliders -row 0 -column 0
grid .f1.keepMe.quit -row 1 -column 0

##########
# Arrange the frames in the GUI window
#
grid .fRGB -row 6
grid .f1 -row 0 -sticky ew
grid .lutSelect -row 3 -sticky ew
grid .imageSelect -row 1 -sticky ew
grid .heightSwitch -row 2 -sticky ew


renWin Render


###
# Removes all actors, and then adds the selected actor
# based on image and desired dimensionality.
#
proc setImage {args} {
	global imageSelection
	global heightSwitch

	ren1 RemoveActor actor_3D_dem
	ren1 RemoveActor actor_3D_mtHood
	ren1 RemoveActor actor_2D_thorax
	ren1 RemoveActor actor_3D_thorax
	ren1 RemoveActor actor_2D_mtHood

	switch $imageSelection	{
		"mtHood"	{switch $heightSwitch	{
					"two"		{
							ren1 AddActor actor_2D_mtHood
							}
					"three"		{
							 ren1 AddActor actor_3D_mtHood
							}
							}
				}
		"thorax"	{switch $heightSwitch	{
					"two"		{ren1 AddActor actor_2D_thorax

							}
					"three"		{
							ren1 AddActor actor_3D_thorax
							}
							}
				}
		"dem"		{ren1 AddActor actor_3D_dem
				set $heightSwitch three
				}
				}			
	resetCamera
}

###
# Resets the camera. 
#
proc resetCamera {args} {
	ren1 ResetCamera
	ren1 ResetCameraClippingRange
	[ren1 GetActiveCamera] OrthogonalizeViewUp
	[ren1 GetActiveCamera] SetViewUp 0 1 0
	renWin Render
}

###
# Resets the RGB and HSV sliders to their default, grayscale values.
#
proc resetDefaults {args} {
	switch $args	{
		"RGB"	{.fRGB.sliders.sstart1 set 0
			.fRGB.sliders.sstart2 set 0
			.fRGB.sliders.sstart3 set 0
			.fRGB.sliders.send1 set 255
			.fRGB.sliders.send2 set 255
			.fRGB.sliders.send3 set 255
			}
		"HSV"	{.fHSV.sliders.sstart1 set 0.0
			.fHSV.sliders.sstart2 set 0.0
			.fHSV.sliders.sstart3 set 0.0
			.fHSV.sliders.send1 set 0.0
			.fHSV.sliders.send2 set 0.0
			.fHSV.sliders.send3 set 1.0
			}
	}
	renWin Render
}

###
# Switches between the RGB and HSV lookup tables
#
proc setLUT {args} {
	global colorSpace
	switch $colorSpace {
	"RGB"	{grid forget .fHSV
		grid .fRGB -row 6
		resetDefaults RGB
		mapper_2D_mtHood SetLookupTable RGBlut
		mapper_3D_mtHood SetLookupTable RGBlut
		mapper_3D_thorax SetLookupTable RGBlut
		mapper_2D_thorax SetLookupTable RGBlut
		mapper_3D_dem SetLookupTable RGBlut
		}
	"HSV"	{grid forget .fRGB
		grid .fHSV -row 6
		resetDefaults HSV
		mapper_2D_mtHood SetLookupTable HSVlut
		mapper_3D_mtHood SetLookupTable HSVlut
		mapper_3D_thorax SetLookupTable HSVlut
		mapper_2D_thorax SetLookupTable HSVlut
		mapper_3D_dem SetLookupTable HSVlut
		}
	}

}

###
# Sets the RGB start and end points for the lookup table,
# and builds the specified table
#
proc setRGB {args} {
 	for {set i 0} {$i <= 255} {incr i} {
    		set R [expr ([.fRGB.sliders.sstart1 get] + (([.fRGB.sliders.send1 get] - [.fRGB.sliders.sstart1 get]) / 255.0 * $i)) / 255.0]
    		set G [expr ([.fRGB.sliders.sstart2 get] + (([.fRGB.sliders.send2 get] - [.fRGB.sliders.sstart2 get]) / 255.0 * $i)) / 255.0]
    		set B [expr ([.fRGB.sliders.sstart3 get] + (([.fRGB.sliders.send3 get] - [.fRGB.sliders.sstart3 get]) / 255.0 * $i)) / 255.0]
    		RGBlut SetTableValue $i $R $G $B 1.0
  		}	
	renWin Render
}

###
# Sets the HSV start and end points for the lookup table,
# and builds the specified table
#
proc setHSV {args} {
	HSVlut SetHueRange [.fHSV.sliders.sstart1 get] [.fHSV.sliders.send1 get]	
	HSVlut SetSaturationRange [.fHSV.sliders.sstart2 get] [.fHSV.sliders.send2 get]
	HSVlut SetValueRange [.fHSV.sliders.sstart3 get] [.fHSV.sliders.send3 get]
	HSVlut Build
	renWin Render
}
